import mongoose from "mongoose";
import Address from "../model/address.model.js";
import Customers from "../model/customer.model.js";
import Location from "../model/location.model.js";
const ObjectId = mongoose.Types.ObjectId;

// Add Address
export const addAddress = async (req, res) => {
    const { location, landmark, city, state, PIN, latitude, longitude, customer_Id, isDefault } = req.body;

    // Check if an address with the same name already exists
    let existingAddress = await Address.findOne({ location: { $regex: new RegExp(`^${location}$`, 'i') }, city: { $regex: new RegExp(`^${city}$`, 'i') }, PIN, customer_Id });
    const customerDetails = await Customers.findById(customer_Id);

    if (existingAddress) {
        if (existingAddress.isDelete) {
            // If existing address is marked as deleted, update fields
            existingAddress.isDelete = false;
            // existingAddress.isActive = true;

            // Set isActive based on locationExists
            const locationExists = await Location.findOne({ PIN, isActive: true });
            existingAddress.isActive = locationExists ? true : false;

            // If customer wants to set this as default, handle accordingly
            if (isDefault) {
                // Set existing address as default
                existingAddress.isDefault = true;

                // Find and update previous default addresses for the customer
                const previousDefaultAddresses = await Address.find({ customer_Id, isDefault: true, _id: { $ne: existingAddress._id } });
                if (previousDefaultAddresses.length > 0) {
                    for (const address of previousDefaultAddresses) {
                        address.isDefault = false;
                        await address.save();
                    }
                }
            }

            await existingAddress.save();

            return res.send({
                "uploaded": true,
                "message": "Address updated successfully.",
                "addressDetails": existingAddress,
                "customerDetails": customerDetails
            });
        } else {
            return res.send({
                "uploaded": false,
                "message": "The address already exists.",
                "addressDetails": existingAddress,
                "customerDetails": customerDetails
            });
        }
    }

    // Input validation for city & state
    try {
        validateInput(req.body.city, "city");
        validateInput(req.body.state, "state");
    } catch (error) {
        return res.status(400).send({
            error: true,
            message: error.message
        });
    }

    // Check if PIN is a valid 6-digit number
    if (!isValidPIN(PIN)) {
        return res.status(400).send({
            error: true,
            message: "Invalid PIN. PIN must be a 6-digit number."
        });
    }

    const locationExists = await Location.findOne({
        PIN,
        isActive: true
    });

    let isActive = locationExists ? true : false; // Set isActive based on locationExists

    let newAddressData = {
        location,
        landmark,
        city,
        state,
        PIN,
        latitude,
        longitude,
        customer_Id,
        isActive
    };

    // If isDefault is explicitly provided and true, or if it's the first address and isDefault is not provided, set it for the new address
    if ((isDefault === true) || (await Address.countDocuments({ customer_Id }) === 0 && isDefault !== false)) {
        newAddressData.isDefault = true;

        // Find and update previous default addresses for the customer
        const previousDefaultAddresses = await Address.find({ customer_Id, isDefault: true });
        if (previousDefaultAddresses.length > 0) {
            for (const address of previousDefaultAddresses) {
                address.isDefault = false;
                await address.save();
            }
        }
    }

    const newAddress = await Address.create(newAddressData);

    if (newAddress) {
        // Only update customer if isActive is true
        if (isActive) {
            const customer = await Customers.findByIdAndUpdate(customer_Id, { $set: { isComplete: true } });

            if (customer) {
                return res.send({
                    error: false,
                    message: "Saved successfully. Profile is now completed.",
                    "addressDetails": newAddress,
                    "customerDetails": customerDetails
                });
            }
        } else {
            return res.send({
                error: false,
                message: "Saved successfully. Profile is incomplete due to inactive location.",
                "addressDetails": newAddress,
                "customerDetails": customerDetails
            });
        }
    }

    return res.status(500).send({
        error: true,
        message: "Failed to save customer address or update customer profile."
    });
}


// Get Address by ID
export const getAddressById = async (req, res) => {
    const addressId = req.params.id;
    const filteredAddress = await Address.findOne({ _id: new ObjectId(addressId), isDelete: false });
    if (filteredAddress) {
        res.send(filteredAddress);
    } else {
        res.status(500).json({ error: 'Address not found' });
    }
}

//Get Address by CustomerId
export const getAddressByCusId = async (req, res) => {
    // const filteredAddress = await Address.find({ customer_Id: new ObjectId(req.params.cusId), isActive: true, isDelete: false })
    //     .sort({ createdAt: -1 }).exec();

    // if (filteredAddress) {
    //     res.send(filteredAddress);
    // } else {
    //     res.status(500).json({ error: 'Address not found' });
    // }
    const cusId = new ObjectId(req.params.cusId);
    const pipeline = [
        {
          $match: {
            customer_Id: req.params.cusId,
            // isActive: true,
            isDelete: false
          }
        },
        // {
        //     $lookup: {
        //         from: 'locations',
        //         let: { pin: "$PIN" }, // Define a variable to store the PIN value from the "Address" collection
        //         pipeline: [
        //             { $match:
        //                 { $expr:
        //                     { $eq: [ "$PIN", "$$pin" ] }
        //                 }
        //             },
        //             { $limit: 1 }
        //         ],
        //         as: 'location1'
        //     }
        //   },
        //   {
        //     $unwind: '$location1'
        //   },
          {
            $project: {
                _id: 1,
                customer_Id: 1,
                isActive: 1,
                isDelete: 1,
                isDefault: 1,
                location: 1,
                city: 1,
                state: 1,
                landmark: 1,
                latitude: 1,
                longitude: 1,
                PIN: 1,
                cityid: 1,
                cityname: 1,
                createdAt: 1 // Include createdAt field in the projection
            }
          },
          {
            $sort: { isDefault: -1, createdAt: -1 }
          }
      ];


      const result = await Address.aggregate(pipeline);

      if (result) {
        //   res.json({
        //       isSuccess: true,
        //       message: "Address",
        //       data: result
        //   });
            console.log(result);
            res.send(result);
      } else {
        //   res.json({
        //       isSuccess: false,
        //       message: "Address not found",
        //       data: []
        //   });
            console.error(err);
            res.status(500).json({ error: 'Address not found' });
      }
      
}


// Update Address By ID
export const updateAddressById = async (req, res) => {
    const { _id, location, landmark, city, state, PIN, latitude, longitude, customer_Id } = req.body;

    // Check if an address with the same name already exists
    let existingAddress = await Address.findOne({ location: { $regex: new RegExp(`^${location}$`, 'i') }, city: { $regex: new RegExp(`^${city}$`, 'i') }, PIN, customer_Id });
    try {
        validateInput(city, "city");
        validateInput(state, "state");
    } catch (error) {
        return res.status(400).send({
            error: true,
            message: error.message
        });
    }

    // Check if PIN is a valid 6-digit number
    if (!isValidPIN(PIN)) {
        return res.status(400).send({
            error: true,
            message: "Invalid PIN. PIN must be a 6-digit number."
        });
    }

    if (existingAddress) {
        if (existingAddress.isDelete) {
            // If existing address is marked as deleted, update fields
            existingAddress.isDelete = false;
            existingAddress.isActive = true;
            await existingAddress.save();

            return res.send({ "uploaded": true, "message": "Address updated successfully." });
        } else {
            return res.send({ "uploaded": false, "message": "The address already exists." });
        }
    } else {
        const updateAddress = await Address.updateOne(
            { _id: new ObjectId(_id) },
            {
                $set: {
                    "location": location,
                    "landmark": landmark,
                    "city": city,
                    "state": state,
                    "PIN": PIN,
                    "latitude": latitude,
                    "longitude": longitude
                }
            }
        );

        if (updateAddress) {
            return res.status(200).json({ status: 200, message: "Update successful." });
        } else {
            return res.status(500).json({ status: 500, message: "Update failed or no changes made." });
        }
    }

};


// Delete address by ID
export const deleteAddress = async (req, res) => {
    const address = await Address.updateOne({ _id: new ObjectId(req.params.id) }, { $set: { "isActive": false, "isDelete": true } });
    if (address) {
        res.status(200).json({ message: "deleted success" });
    } else {
        res.status(500).json({ message: "deletion failed" });
    }
}


//Get all (except deleted) Address of a Customer
export const viewAddressByCustomerId = async (req, res) => {
    const filteredAddress = await Address.find({ customer_Id: new ObjectId(req.params.cusId), isDelete: false })
    .sort({ createdAt: -1 }).exec();
    
    console.log("find address");
    if (filteredAddress) {
        console.log("true address");
        res.status(200).json({
            error: false,
            data : filteredAddress
        });
    } else {
        res.status(500).json({ error: 'Address not found' });
    }
}


//Get all (except deleted) Address of a Customer     //commented as it write twice 
// export const viewAddressByCustomerId = async (req, res) => {
//     const filteredAddress = await Address.find({ customer_Id: new ObjectId(req.params.cusId), isDelete: false })
//     .sort({ createdAt: -1 }).exec();
    
//     console.log("find address");
//     if (filteredAddress) {
//         console.log("true address");
//         res.status(200).json({
//             error: false,
//             data : filteredAddress
//         });
//     } else {
//         res.status(500).json({ error: 'Address not found' });
//     }
// }


// FUNCTIONS

// Input validation for city & state
function validateInput(input, fieldName) {
    if (!/^[A-Za-z\s]+$/.test(input)) {
        throw new Error(`Invalid characters in ${fieldName}. Only alphabets and spaces allowed.`);
    }
}

// Check if PIN is a valid 6-digit number
function isValidPIN(PIN) {
    // Check if PIN is a 6-digit number
    const pinRegex = /^\d{6}$/;
    return pinRegex.test(PIN);
}


// Get service available addresses only of a customer
export const serviceableAddress = async (req, res) => {
    const { location, city, state, PIN} = req.body;

    try {
        // Check if the service is available at the provided address
        const serviceLocation = await Location.findOne({
            PIN,
            isActive: true
        });

        if (serviceLocation) {
            // Service is available at the address
            return res.json({ uploaded: true, message: 'Service is available at this address' });
        } else {
            // Check if the non-serviceable location already exists
            let nonServiceableLocation = await Location.findOne({
                PIN,
                isActive: false
            });

            if (!nonServiceableLocation) {
                // If the location doesn't exist as non-serviceable, create it
                const newLocation = new Location({
                    PIN,
                    isActive: false, // Non-serviceable
                    count: 1 // Set count to 1 for a new non-serviceable location
                });

                await newLocation.save();
                return res.json({
                    error: false,
                    uploaded: false,
                    message: 'Service is not available at this address'
                });
            } else {
                // If the same non-serviceable location already exists, increment the count
                nonServiceableLocation.count += 1;
                await nonServiceableLocation.save();
                return res.json({
                    error: false,
                    uploaded: false,
                    message: 'Service is not available at this address'
                });
            }
        }
    } catch (error) {
        console.error(error);
        res.status(500).json({ error: 'Internal Server Error' });
    }
};

// Set Default Address
export const setDefaultAddress = async (req, res) => {
    const addressId = req.params.id;

    try {
        // Find the address by ID
        const address = await Address.findById(addressId);
        if (!address) {
            return res.status(404).json({ error: 'Address not found' });
        }

        // Find and update previous default addresses for the customer
        const previousDefaultAddresses = await Address.find({ customer_Id: address.customer_Id, isDefault: true });
        console.log("previousDefaultAddresses", previousDefaultAddresses);
        if (previousDefaultAddresses.length > 0) {
            for (const prevAddress of previousDefaultAddresses) {
                prevAddress.isDefault = false;
                await prevAddress.save();
            }
        }

        // Set the current address as default
        address.isDefault = true;
        await address.save();

        res.json({ message: 'Default address set successfully', address });
    } catch (error) {
        console.error(error);
        res.status(500).json({ error: 'Internal Server Error' });
    }
};
