import mongoose from "mongoose";
import NGO from '../model/ngo.model.js';
import Donation from '../model/donation.model.js';
import Order from '../model/order.model.js';
import multer from "multer";
import axios from 'axios';
import dotenv from 'dotenv';
import Customers from "../model/customer.model.js";
const ObjectId = mongoose.Types.ObjectId;

dotenv.config();
const apiKey = process.env.APIKEY;
// const apiKey = 'rzp_live_c8RouP3eKq6D1C';
const apiSecret = process.env.APISECRET;
// const apiSecret = '63WBO3yFwX6gHymOiypMjXm7';

const storage = multer.diskStorage({
    destination: 'uploads',
    filename: (req, file, cb) => {
        cb(null, Date.now() + "_" + file.originalname.trim());
    }
});
const upload = multer({
    storage: storage
}).single('certificateUrl');


// Check if customer has PAN or not
export const checkPAN = async (req, res) => {
    const customerId = new ObjectId(req.params.id);

    try {
        const customer = await Customers.findById(customerId);
        
        if (customer && customer.PAN) {
            return res.send({ hasPAN: true, data: customer });
        } else {
            return res.send({ hasPAN: false });
        }
    } catch (error) {
        console.error(error);
        return res.send({ message: 'Internal Server Error' });
    }
};

// Function to validate PAN format
function validatePANFormat(pan) {
    const panRegex = /[A-Z]{5}[0-9]{4}[A-Z]{1}/;
    return panRegex.test(pan);
}

// To update PAN of a customer
export const updatePAN = async (req, res) => {
    const customerId = new ObjectId(req.params.custId);
    const { PAN } = req.body;

    try {
        // Validate PAN format
        if (!validatePANFormat(PAN)) {
            return res.send({
                isSuccess: false,
                message: 'Invalid PAN format'
            });
        }

        await Customers.findByIdAndUpdate(customerId, { PAN });
        const updatedCustomer = await Customers.findById(customerId);

        return res.send({
            isSuccess: true,
            message: 'PAN number updated successfully',
            customer: updatedCustomer
        });
    } catch (error) {
        console.error(error);
        return res.send({
            isSuccess: false,
            message: 'Internal Server Error'
        });
    }
};


// Update certificate URL and status
export const uploadCertificate = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
            return res.send({
                error: true,
                message: "File upload failed"
            });
        }

        try {
            const donation = await Donation.findOne({ orderId: new ObjectId(req.body.orderId) });

            if (!donation) {
                return res.send({ isSuccess: false, message: 'Donation data not found' });
            }

            if (req.file) {
                const certificateUrl = `https://devapisbot.ceibagreen.com/uploads/${req.file.filename}`;
                donation.certificateUrl = certificateUrl;
                donation.certificate_status = 'issued';
            }

            const savedDonation = await donation.save();

            if (savedDonation) {
                return res.send({ isSuccess: true, message: 'Certificate URL updated successfully', data: savedDonation });
            } else {
                return res.send({ isSuccess: false, message: 'Failed to update the donation' });
            }
        } catch (error) {
            console.error(error);
            return res.send({ success: false, message: 'An error occurred while updating certificateUrl' });
        }
    });
};


// To fetch donation history
export const donationHistory = async (req, res) => {
    try {
        const { customerId } = req.params;
        
        // Find donations and populate related fields
        const donations = await Donation.find({ customerId })
            .populate('NGO_id', 'name')
            .populate('orderId', 'order_id status') // Populate status of order
            .select('donationDate amount customerId orderId donationStatus certificateUrl certificate_status')
            .sort({ createdAt: -1 });
        console.log("donations", donations);

        if (donations.length > 0) {
            // Fetch the customer's PAN
            const customer = await Customers.findById(customerId).select('PAN');

            const filteredDonations = donations.filter(donation => {
                return donation.orderId && donation.orderId.status === 'processed';
            });

            if (filteredDonations.length > 0) {
                const missingNGOs = filteredDonations.some(donation => donation.NGO_id == null);
                const missingOrders = filteredDonations.some(donation => donation.orderId == null);

                if (missingNGOs) {
                    res.send({ message: 'NGO information not found for some donations.' });
                    return;
                }

                if (missingOrders) {
                    res.send({ message: 'Order information not found for some donations.' });
                    return;
                }

                const formattedDonations = filteredDonations.map(donation => ({
                    organisationName: donation.NGO_id.name,
                    customerPAN: customer.PAN,
                    donationDate: donation.donationDate,
                    donationAmount: donation.amount,
                    certificateStatus: donation.certificate_status,
                    certificateUrl: donation.certificateUrl || null,
                    orderId: donation.orderId.order_id
                }));

                console.log("formattedDonations", formattedDonations);

                res.send({ isSuccess: true, data: formattedDonations });
            } else {
                res.send({ message: 'No processed donations found for the provided customer ID' });
            }
        } else {
            res.send({ message: 'No donations found for the provided customer ID' });
        }
    } catch (error) {
        console.error('Error fetching donations:', error);
        res.send({ message: 'Internal server error' });
    }
};

// To fetch donation certificates for a customer
export const getCertificate = async (req, res) => {
    try {
        const customerId = req.params.cusId;
        const orderId = req.params.ordId;

        console.log("customerId", customerId);
        console.log("order id: ", orderId);
        const donation = await Donation.findOne({ customerId: customerId, orderId: orderId });
        const certificates = donation.certificateUrl;
        if (certificates && certificates != "null") {
            res.send({ isSuccess: true, msg: "Certificate found successfully", certificate: certificates, data: donation });
        } else {
            res.send({ isSuccess: false, msg: "Certificate not exists for that order", data: donation });
        }
    } catch (error) {
        console.error('Error fetching certificates:', error);
        res.send({ message: 'Internal server error' });
    }
};
