import Customers from '../model/customer.model.js';
import Plogging from '../model/plogging.model.js';
import { pushNotificationForEvents } from './notification.controller.js';
import multer from "multer";
// const ObjectId = mongoose.Types.ObjectId;

const Storage = multer.diskStorage({
    destination: 'uploads',
    filename: (req, res, cb) => {
        cb(null, Date.now() + "_" + (res.originalname).trim());
    }
});

const upload = multer({
    storage: Storage,
}).single('image');

// Adding plogging details
export const addPloggingData = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log("Error occurred during file upload:", err);
            return res.send({
                error: true,
                message: "File upload failed"
            });
        }

        const { title, eventid, date, startTime, endTime, location, cityid, about, plg_id, plg_name, plg_mail, plg_contact, status } = req.body;

        try {
            // Check if plogging data with the same title, eventid, date & plg_id already exists
            const existingPlogging = await Plogging.findOne({ title, eventid, date, plg_id });
            if (existingPlogging) {
                console.log("Plogging data already exists:", existingPlogging);
                return res.send({
                    isSuccess: false,
                    message: "Plogging data with the same title, eventid, date & plg_id already exists."
                });
            }

            console.log("No existing plogging data found. Proceeding to save new data...");

            let plogging_data = new Plogging({
                title,
                eventid,
                date,
                startTime,
                endTime,
                location,
                cityid,
                about,
                plg_id,
                plg_name,
                plg_mail,
                plg_contact,
                status
            });

            if (req.file) {
                plogging_data.image = {
                    contentType: 'image',
                    data: req.file.filename
                };
                plogging_data.imageURL = 'uploads/' + req.file.filename;
            }
            await plogging_data.save();
            console.log("Plogging data saved:", plogging_data);

            // Send notification to customers of the specified city
        try {
            pushNotificationForEvents(plogging_data);
          } catch (error) {
            console.error("Error sending notifications:", error);
          }
    
          res.send({ "uploaded": true, data: plogging_data });
        } catch (error) {
              // console.error(error);
              res.send({ "uploaded": false, error: error });
          }
      });
  };


// Get all Plogging Event details
export const getAllEvents = async (req, res) => {
    const events = await Plogging.find({ isDelete: false })
        .sort({ createdAt: -1 }).exec();

    if (events) {
        res.send(events);
    } else {
        res.status(500).json({ error: 'Plogging Events not found' });
    }
}


// Get Plogging Event details by ID
export const getEventById = async (req, res) => {
    const events = await Plogging.find({ _id: (req.params.id), isDelete: false });

    if (events.length > 0) {
        res.send({
            isSuccess: true,
            "Event Details": events
        });
    } else {
        res.send({ msg: 'The Plogging Event not found' });
    }
}

// Register customers for an event
export const registerCustomer = async (req, res) => {
    const { eventid, customerid } = req.body;

    try {
        // Check if the plogging event with the specified eventid exists
        let existingPlogging = await Plogging.findOne({ eventid });

        if (!existingPlogging) {
            return res.send({
                isSuccess: false,
                message: "Plogging event not found."
            });
        }

        // Check if the customer already registered for the event
        if (existingPlogging.customers.includes(customerid)) {
            return res.send({
                isSuccess: false,
                message: "Customer with the same customerid is already registered for the event."
            });
        }

        // Add the customerid to the customers array of the existing plogging event
        existingPlogging.customers.push(customerid);

        // Update the customers array in the existing plogging event
        await Plogging.updateOne({ eventid }, { $push: { customers: customerid } });

        // Fetch customer details based on customerid
        const customer = await Customers.findById(customerid);

        if (!customer) {
            return res.send({
                isSuccess: false,
                message: "Customer not found."
            });
        }

        return res.send({
            isSuccess: true,
            message: "Customer registered successfully for the event.",
            data: {
                ploggingEvent: existingPlogging,
                customer: customer
            }
        });
    } catch (error) {
        console.error(error);
        return res.send({
            isSuccess: false,
            message: "Something went wrong."
        });
    }
};

// Check whether the customer already registered for the Event or not!
export const checkCustomerRegistration = async (req, res) => {
    const { eventid, customerid } = req.body;

    try {
        // Check if the plogging event with the specified eventid exists
        const existingPlogging = await Plogging.findOne({ eventid });

        if (!existingPlogging) {
            return res.send({
                isSuccess: false,
                message: "Plogging event not found."
            });
        }

        // Check if the customer already registered for the event
        if (existingPlogging.customers.includes(customerid)) {
            return res.send({
                isSuccess: true,
                message: "Customer already registered for the event.",
                isRegistered: true
            });
        } else {
            return res.send({
                isSuccess: true,
                message: "Customer is not registered for the event.",
                isRegistered: false
            });
        }

    } catch (error) {
        console.error(error);
        return res.send({
            isSuccess: false,
            message: "Something went wrong."
        });
    }
};


// GET all customers registered for a specific plogging event
export const RegCustByEventId = async (req, res) => {
    try {
        // Find the plogging event by ID
        const ploggingEvent = await Plogging.findById(req.params.id)
            .populate('customers', '-__v -createdAt -updatedAt');

        // Check if the plogging event exists and has registered customers
        if (ploggingEvent && ploggingEvent.customers.length > 0) {
            res.status(200).send({
                isSuccess: true,
                eventTitle: ploggingEvent.title,
                registeredCustomers: ploggingEvent.customers
            });
        } else {
            res.status(404).send({ msg: 'No registered customers found for this Plogging Event' });
        }
    } catch (error) {
        console.error(error);
        res.status(500).send({ msg: 'Internal Server Error' });
    }
};